@@ -68,53 +68,7 @@ def expression2qualitative(genes, column_list=[], proportion=0.25, method="keep"
     return genes
 
 
-def prepare_expr_split_gen_list(rxn, modelname):
-    """
-
-    Parameters
-    ----------
-    rxn: cobra.Reaction
-    modelname: str
-        The name of the model. Currently only supports human1, recon1, recon2, iMM1865, zebrafish1
-
-    Returns
-    -------
-
-    """
-    if modelname == "recon2":
-        expr_split = rxn.gene_reaction_rule.replace("(", "( ").replace(")", " )").split()
-        expr_split = [s.replace(':', '_') if ':' in s else s for s in expr_split]
-        rxngenes = re.sub('and|or|\(|\)', '', rxn.gene_reaction_rule).split()
-        gen_list = set([s.replace(':', '_') for s in rxngenes if ':' in s])
-    elif modelname == "recon1":
-        expr_split = rxn.gene_reaction_rule.replace("(", "( ").replace(")", " )").split()
-        expr_split = ["g_" + s[:-4] if '_' in s else s for s in expr_split]
-        rxngenes = re.sub('and|or|\(|\)', '', rxn.gene_reaction_rule).split()
-        gen_list = set(["g_" + s[:-4] for s in rxngenes if '_' in s])
-    elif modelname == "iMM1865":
-        expr_split = rxn.gene_reaction_rule.split()
-        expr_split = ["g_" + s if s.isdigit() else s for s in expr_split]
-        rxngenes = re.sub('and|or|\(|\)', '', rxn.gene_reaction_rule).split()
-        gen_list = set(["g_" + s for s in rxngenes if s.isdigit()])
-    elif modelname == "human1":
-        expr_split = rxn.gene_reaction_rule.replace("(", "( ").replace(")", " )").split()
-        gen_list = set([g.id for g in rxn.genes])
-    elif modelname == "zebrafish1":
-        expr_split = rxn.gene_reaction_rule.replace("(", "( ").replace(")", " )").split()
-        expr_split = [re.sub(':|\.|-', '_', s) for s in expr_split]
-        gen_list = set([re.sub(':|\.|-', '_', g.id) for g in rxn.genes])
-    elif modelname == "test":
-        expr_split = [rxn.gene_reaction_rule]
-        gen_list = set([g.id for g in rxn.genes])
-    else:
-        print("Modelname not found")
-        expr_split = None
-        gen_list = None
-
-    return expr_split, gen_list
-
-
-def apply_gpr(model, gene_weights, modelname, save=True, filename="reaction_weights"):
+def apply_gpr(model, gene_weights, save=True, filename="reaction_weights", duplicates="remove"):
     """
     Applies the GPR rules from a given metabolic model for creating reaction weights
 
@@ -122,29 +76,41 @@ def apply_gpr(model, gene_weights, modelname, save=True, filename="reaction_weig
     ----------
     model: cobra.Model
         a cobrapy model
-    gene_weights: dict
-        a dictionary containing gene IDs & weights
-    modelname: str
-        the name of the model
+    gene_weights: dict or pd.Series
+        a dictionary of pandas Series containing gene IDs & weights
     save: bool
         if True, saves the reaction weights as a csv file
-
+    filename: str
+        path where the file will be saved
+    duplicates: str, any of "remove", "max", "min", "mean", "median"
+        determines how to deal with genes presenting several expression values
     Returns
     -------
     reaction_weights: dict where keys = reaction IDs and values = weights
     """
+    operations = {"min": np.min, "max": np.max, "mean": np.mean, "median": np.median,
+                  "remove": lambda x: x.mean() if len(x.value_counts()) == 1 else 0.}
+    if type(gene_weights) == pd.Series:
+        for gene in set(gene_weights.index):
+            if type(gene_weights[gene]) == pd.Series:
+                vals = gene_weights.pop(gene)
+                gene_weights[gene] = operations[duplicates](vals)
+        gene_weights = gene_weights.to_dict()
+
     reaction_weights = {}
+    gene_weights = {str(k): float(v) for k, v in gene_weights.items()}
     for rxn in model.reactions:
         if len(rxn.genes) > 0:
-            expr_split, gen_list = prepare_expr_split_gen_list(rxn, modelname)
-            new_weights = {g: float(gene_weights.get(g, 0)) for g in gen_list}
+            gen_list = [g.id for g in rxn.genes]
+            expr_split = rxn.gene_reaction_rule.replace("(", "( ").replace(")", " )").split()
+            expr_split = ["g_" + re.sub(':|\.|-', '_', s) if s in gen_list else s for s in expr_split]
+            new_weights = {"g_" + re.sub(':|\.|-', '_', g): gene_weights.get(g, 0) for g in gen_list}
             negweights = []
             for g, v in new_weights.items():
                 if v < 0:
                     new_weights[g] = -v - 1e-15
                     negweights.append(-v)
             expression = ' '.join(expr_split).replace(' or ', ' * ').replace(' and ', ' + ')
-            # weight = sympify(expression).xreplace({Mul: Max}).xreplace({Add: Min})
             weight = replace_MulMax_AddMin(sympify(expression)).subs(new_weights)
             if weight + 1e-15 in negweights:
                 weight = -weight - 1e-15
@@ -162,19 +128,18 @@ if __name__ == "__main__":
 
     parser = argparse.ArgumentParser(description=description, formatter_class=argparse.RawTextHelpFormatter)
     parser.add_argument("-m", "--model", help="GEM in json, sbml or matlab format")
-    parser.add_argument("-n", "--modelname", help="supported: human1, recon1, recon2, iMM1865, zebrafish1")
     parser.add_argument("-g", "--gene_file", help="csv file containing gene identifiers and scores")
     parser.add_argument("-o", "--output", default="reaction_weights",
                         help="Path to which the reaction_weights .csv file is saved")
     parser.add_argument("--gene_ID", default="ID", help="column containing the gene identifiers")
     parser.add_argument("--gene_score", default="t", help="column containing the gene scores")
+    parser.add_argument("-d", "--duplicates", default="remove", help="column containing the gene scores")
     parser.add_argument("--convert", action='store_true', help="converts gene expression to qualitative weights")
     parser.add_argument("-t", "--threshold", type=float, default=.25,
                         help="proportion of genes that are highly/lowly expressed (only used if --convert is selected)")
     args = parser.parse_args()
 
     model = read_model(args.model)
-    model_list = ["human1", "recon1", "recon2", "iMM1865", "zebrafish1"]
 
     genes = pd.read_csv(args.gene_file).set_index(args.gene_ID)
     if args.convert:
@@ -182,15 +147,5 @@ if __name__ == "__main__":
                                        outpath=args.output+"_qual_geneweights")
     gene_weights = pd.Series(genes[args.gene_score].values, index=genes.index)
 
-    # current behavior: all genes with several different weights are removed
-    for x in set(gene_weights.index):
-        if type(gene_weights[x]) != np.float64:
-            if len(gene_weights[x].value_counts()) > 1:
-                gene_weights.pop(x)
-    gene_weights = gene_weights.to_dict()
-
-    if args.modelname not in model_list:
-        print("Unsupported model. The currently supported models are: human1, recon1, recon2, iMM1865, zebrafish1")
-    else:
-        reaction_weights = apply_gpr(model=model, gene_weights=gene_weights, modelname=args.modelname, save=True,
-                                     filename=args.output)
+    reaction_weights = apply_gpr(model=model, gene_weights=gene_weights, save=True, filename=args.output,
+                                 duplicates=args.duplicates)
